/*
 * Caller ID with faux nixie clock on a 480x320 pixel LCD
 */
#include <stdio.h>
#include <time.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_log.h"
#include "audio_task.h"
#include "cid_task.h"
#include "gui_task.h"
#include "ring_task.h"
#include "ntp.h"
#include "wifi.h"


static const char* TAG = "main";

void app_main(void)
{
	ESP_LOGI(TAG, "nixie_caller_id starting");
	
	// Start the tasks that actually comprise the application
	//   Core 0 : PRO
    //   Core 1 : APP
    //
    xTaskCreatePinnedToCore(&audio_task, "audio_task", 3072, NULL, tskIDLE_PRIORITY+3, &task_handle_audio, 1);
    xTaskCreatePinnedToCore(&cid_task, "cid_task", 3072, NULL, tskIDLE_PRIORITY+3, &task_handle_cid, 0);
    xTaskCreatePinnedToCore(&gui_task, "gui_task", 2560, NULL, tskIDLE_PRIORITY+1, &task_handle_gui, 0);
    xTaskCreatePinnedToCore(&ring_task, "ring_task", 2048, NULL, tskIDLE_PRIORITY+2, &task_handle_ring, 0);
    
    // Setup our timezone
    timezone_set("MST7MDT,M3.2.0,M11.1.0");
    
    // Start Wifi and try to get the time
    if (wifi_connect() == ESP_OK) {
    	ntp_start();
    	ntp_wait_for_sync();
    }
}
